#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include <limits>
#include <algorithm>
#include <sstream>
#include "player.hpp"

void draw(sf::RenderWindow &window, Player& player, sf::Sprite& sea, sf::Text& messageText, sf::Text& messageText2, sf::Text& messageText3, sf::Text& bottomLeftText, sf::Text& bottomRightText, sf::Sprite& sunset, sf::Sprite& reflection, sf::Sprite& seaTop, sf::Sprite& seaEffect, sf::Sprite& start);

namespace patch
{
	template < typename T > std::string to_string( const T& n )
	{
		std::ostringstream stm ;
		stm << n ;
		return stm.str() ;
	}
}

int main()
{
	sf::Vector2f startPosition(0, 4400);
	sf::ContextSettings settings;
	settings.antialiasingLevel = 8;

	// create the window
	sf::RenderWindow window(sf::VideoMode(800, 600), "Beneath the Surface");

	sf::Font font;
	font.loadFromFile("./assets/ocean_coastlines.otf");
	sf::Text messageText;
	messageText.setFont(font);
	messageText.setColor(sf::Color(255,255,255));
	messageText.setStyle(sf::Text::Style::Bold);
	messageText.rotate(-30);
	messageText.setCharacterSize(60);

	sf::Text messageText2;
	messageText2.setFont(font);
	messageText2.setColor(sf::Color(255,255,255));
	messageText2.setStyle(sf::Text::Style::Bold);
	messageText2.rotate(30);
	messageText2.setCharacterSize(60);

	sf::Text messageText3;
	messageText3.setFont(font);
	messageText3.setColor(sf::Color(255,255,255));
	messageText3.setStyle(sf::Text::Style::Bold);
	messageText3.setCharacterSize(100);

	sf::Font bottomFont;
	bottomFont.loadFromFile("./assets/ocean.ttf");
	sf::Text bottomRightText;
	bottomRightText.setFont(bottomFont);
	bottomRightText.setColor(sf::Color(255,255,255));
	bottomRightText.setStyle(sf::Text::Style::Bold);
	bottomRightText.setCharacterSize(60);

	sf::Text bottomLeftText;
	bottomLeftText.setFont(bottomFont);
	bottomLeftText.setColor(sf::Color(255,255,255));
	bottomLeftText.setStyle(sf::Text::Style::Bold);
	bottomLeftText.setCharacterSize(60);

	sf::Texture playerTexture;
	playerTexture.loadFromFile("./assets/player.png");
	Player player(startPosition, playerTexture);

	sf::Texture seaTexture;
	seaTexture.loadFromFile("./assets/sea.png");
	seaTexture.setRepeated(true);
	sf::Sprite sea(seaTexture, sf::IntRect(0,0,200000,100000));
	sea.move(-sea.getLocalBounds().width/2, 200);

	sf::Texture seaTopTexture;
	seaTopTexture.loadFromFile("./assets/sea_top.png");
	seaTopTexture.setRepeated(true);
	sf::Sprite seaTop(seaTopTexture, sf::IntRect(0,0,200000,600));
	seaTop.move(-seaTop.getLocalBounds().width/2, 0);

	sf::Texture seaEffectTexture;
	seaEffectTexture.loadFromFile("./assets/sea_effect.png");
	sf::Sprite seaEffect(seaEffectTexture, sf::IntRect(0,0,1600,1200));

	sf::Texture sunsetTexture;
	sunsetTexture.loadFromFile("./assets/sunset.png");
	sf::Sprite sunset(sunsetTexture, sf::IntRect(0,0,1024,768));
	sunset.move(-sunset.getLocalBounds().width/2+400, -200);

	sf::Texture reflectionTexture;
	reflectionTexture.loadFromFile("./assets/reflection.png");
	sf::Sprite reflection(reflectionTexture, sf::IntRect(0,0,1024,384));
	reflection.move(-reflection.getLocalBounds().width/2+400, reflection.getLocalBounds().height);

	sf::Texture startTexture;
	startTexture.loadFromFile("./assets/start_screen.png");
	sf::Sprite start(startTexture, sf::IntRect(0,0,800,600));
	start.setOrigin(startTexture.getSize().x/2, startTexture.getSize().y/2);
	start.setPosition(startPosition);

	sf::Music music;
	music.openFromFile("./assets/soundtrack1.ogg");
	music.setLoop(true);
	music.setVolume(50);
	music.play();
	/*
	sf::Music music2;
	music2.openFromFile("./assets/soundtrack2.ogg");
	music2.setLoop(true);
	music2.play();
	music2.setVolume(0);
	*/

	int score = 0;
	sf::Clock gameTime;
	std::vector<std::pair<std::string, sf::Time>> messages;
	std::vector<std::pair<std::string, sf::Time>> messages2;

	sf::Clock clock;
	// run the program as long as the window is open
	while (window.isOpen())
	{
		sf::Time timeRemaining = (sf::seconds(180) - gameTime.getElapsedTime());
		if(timeRemaining.asSeconds() <= 0)
		{
			sf::Event event;
			while (window.pollEvent(event))
			{
				// "close requested" event: we close the window
				if(event.type == sf::Event::Closed ||
						(event.type == sf::Event::KeyPressed &&
						 event.key.code == sf::Keyboard::Escape)
				  )
				{
					window.close();
				}
			}
			if(sf::Keyboard::isKeyPressed(sf::Keyboard::R))
			{
				messageText3.setString("");
				player = Player(startPosition, playerTexture);
				score = 0;
				gameTime.restart();
				continue;
			}
			messageText.setString("");
			messageText2.setString("");
			messageText3.setString("Your score was " + patch::to_string(score));
			bottomLeftText.setString("Press R to play again!");
			bottomRightText.setString("");
			draw(window, player, sea, messageText, messageText2, messageText3, bottomLeftText, bottomRightText, sunset, reflection, seaTop, seaEffect, start);
			continue;
		}
		// check all the window's events that were triggered since the last iteration of the loop
		sf::Event event;
		sf::Time elapsed = clock.restart();
		while (window.pollEvent(event))
		{
			// "close requested" event: we close the window
			if(event.type == sf::Event::Closed ||
					(event.type == sf::Event::KeyPressed &&
					 event.key.code == sf::Keyboard::Escape)
			  )
			{
				window.close();
			}
		}
		if(sf::Keyboard::isKeyPressed(sf::Keyboard::Down))
		{
			player.decreaseSpeed();
		}
		if(sf::Keyboard::isKeyPressed(sf::Keyboard::Up))
		{
			player.increaseSpeed();
		}
		if(sf::Keyboard::isKeyPressed(sf::Keyboard::Left))
		{
			player.left(elapsed);
		}
		else if(sf::Keyboard::isKeyPressed(sf::Keyboard::Right))
		{
			player.right(elapsed);
		}
		else
		{
			player.forward();
		}
		player.tick(elapsed, messages, messages2, score);

		for(auto& message: messages)
		{
			message.second += elapsed;
		}
		auto removable = std::remove_if(messages.begin(), messages.end(), [](std::pair<std::string, sf::Time> message){return message.second.asMilliseconds() > 2000;});
		messages.erase(removable, messages.end());
		std::string totalString = "";
		for(const auto& message: messages)
		{
			totalString += message.first;
			totalString += "\n";
		}
		messageText.setString(totalString);

		for(auto& message: messages2)
		{
			message.second += elapsed;
		}
		auto removable2 = std::remove_if(messages2.begin(), messages2.end(), [](std::pair<std::string, sf::Time> message){return message.second.asMilliseconds() > 2000;});
		messages2.erase(removable2, messages2.end());
		std::string totalString2 = "";
		for(const auto& message: messages2)
		{
			totalString2 += message.first;
			totalString2 += "\n";
		}
		messageText2.setString(totalString2);

		if(player.isUnderwater())
		{
			music.setVolume(20.0 + player.getSpeed()*40.0);
		}
		/*
		if(player.isUnderwater())
		{
			music.setVolume(100);
			music2.setVolume(0);
		}
		else
		{
			music.setVolume(0);
			music2.setVolume(100);
		}
		*/

		bottomRightText.setString(
				patch::to_string((int)timeRemaining.asSeconds()/60) +
				":" + (((int)timeRemaining.asSeconds())%60<10?"0":"") +
				patch::to_string((int)timeRemaining.asSeconds()%60)
				);
		bottomLeftText.setString(
				patch::to_string(score)
				);

		float colorMultiplier = 1;
		if(player.getPosition().y < 1500 && player.getPosition().y > 0)
		{
			colorMultiplier = (player.getPosition().y/1500.0f);
		}
		else if(player.getPosition().y <= 0)
		{
			colorMultiplier = 0;
		}
		messageText.setColor(sf::Color(255.0f*colorMultiplier,255.0f*colorMultiplier,255.0f*colorMultiplier));
		messageText2.setColor(sf::Color(255.0f*colorMultiplier,255.0f*colorMultiplier,255.0f*colorMultiplier));
		messageText3.setColor(sf::Color(255.0f*colorMultiplier,255.0f*colorMultiplier,255.0f*colorMultiplier));
		bottomLeftText.setColor(sf::Color(255.0f*colorMultiplier,255.0f*colorMultiplier,255.0f*colorMultiplier));
		bottomRightText.setColor(sf::Color(255.0f*colorMultiplier,255.0f*colorMultiplier,255.0f*colorMultiplier));

		draw(window, player, sea, messageText, messageText2, messageText3, bottomLeftText, bottomRightText, sunset, reflection, seaTop, seaEffect, start);
	}

	return 0;
}

void draw(sf::RenderWindow &window, Player& player, sf::Sprite& sea, sf::Text& messageText, sf::Text& messageText2, sf::Text& messageText3, sf::Text& bottomLeftText, sf::Text& bottomRightText, sf::Sprite& sunset, sf::Sprite& reflection, sf::Sprite& seaTop, sf::Sprite& seaEffect, sf::Sprite& start)
{
	sf::View gameView(sf::FloatRect(0, 0, 800, 600));
	sf::View overlayView(sf::FloatRect(0, 0, 800, 600));
	sf::View backgroundView(sf::FloatRect(0, 0, 800, 600));
	if(player.getPosition().y >= 0.0) //below water
	{
		gameView.setCenter(player.getPosition());
	}
	else //above water
	{
		//gameView.zoom(-10-player.getPosition().y);
		float distance = (0 -  player.getPosition().y)* 1.5;
		if(distance > 600)
		{
			gameView.setSize(distance * 8.0/6.0, distance);
			gameView.setCenter((player.getPosition()+sf::Vector2f(player.getPosition().x,0.0))/2.0f);
		}
		else
		{
			gameView.setCenter((player.getPosition()+sf::Vector2f(player.getPosition().x,0.0))/2.0f);
		}
	}
	// clear the window with black color
	window.clear(sf::Color::Black);
	//draw the sunset:
	window.setView(backgroundView);
	window.draw(sunset);
	if(!player.isUnderwater())
	{
		float reflectionScale = 0;
		reflectionScale = 0.3+(-player.getPosition().y/2000.0);
		reflection.setScale(1, reflectionScale);
		window.draw(reflection);
	}

	window.setView(gameView);
	//draw the sea:
	sf::Vertex seaSurface[] =
	{
		sf::Vertex(sf::Vector2f( 100000,   0), sf::Color(220, 50, 50)),
		sf::Vertex(sf::Vector2f(-100000,   0), sf::Color(220, 50, 50)),
		sf::Vertex(sf::Vector2f(-100000, 400), sf::Color( 50, 80,128)),
		sf::Vertex(sf::Vector2f( 100000, 400), sf::Color( 50, 80,128)),
	};
	window.draw(seaSurface, 4, sf::Quads);
	sf::Vertex seaBottom[] =
	{
		sf::Vertex(sf::Vector2f( 100000,   400), sf::Color( 50, 80,128)),
		sf::Vertex(sf::Vector2f(-100000,   400), sf::Color( 50, 80,128)),
		sf::Vertex(sf::Vector2f(-100000, 10000), sf::Color(  0,  0,  0)),
		sf::Vertex(sf::Vector2f( 100000, 10000), sf::Color(  0,  0,  0)),
	};
	window.draw(seaBottom, 4, sf::Quads);
	sf::Vertex seaDeep[] =
	{
		sf::Vertex(sf::Vector2f( 100000, 10000), sf::Color(  0,  0,  0)),
		sf::Vertex(sf::Vector2f(-100000, 10000), sf::Color(  0,  0,  0)),
		sf::Vertex(sf::Vector2f(-100000,100000), sf::Color(  0,  0,  0)),
		sf::Vertex(sf::Vector2f( 100000,100000), sf::Color(  0,  0,  0)),
	};
	window.draw(seaDeep, 4, sf::Quads);
	window.draw(seaTop);
	seaEffect.setPosition(player.getPosition().x-seaEffect.getLocalBounds().width/2,0);
	window.draw(seaEffect);
	window.draw(sea);
	window.draw(start);

	window.draw(player);

	window.setView(overlayView);
	messageText.setOrigin(messageText.getLocalBounds().width/2.0f, messageText.getLocalBounds().height/2.0f);
	messageText.setPosition(messageText.getGlobalBounds().width/2.0f, messageText.getGlobalBounds().height/2.0f);
	window.draw(messageText);

	messageText2.setOrigin(messageText2.getLocalBounds().width/2.0f, messageText2.getLocalBounds().height/2.0f);
	messageText2.setPosition(800-messageText2.getGlobalBounds().width/2.0f, messageText2.getGlobalBounds().height/2.0f);
	window.draw(messageText2);

	messageText3.setOrigin(messageText3.getLocalBounds().width/2.0f, messageText3.getLocalBounds().height/2.0f);
	messageText3.setPosition(400, 350);
	window.draw(messageText3);

	bottomLeftText.setOrigin(0, bottomLeftText.getLocalBounds().height);
	bottomLeftText.setPosition(10, 570);
	window.draw(bottomLeftText);

	bottomRightText.setOrigin(bottomRightText.getLocalBounds().width, bottomRightText.getLocalBounds().height);
	bottomRightText.setPosition(790, 570);
	window.draw(bottomRightText);

	// end the current frame
	window.display();
}
