#include "player.hpp"

#define M_PI 3.14159265358979323846

Player::Player(sf::Vector2f position, sf::Texture& texture)
{
	this->texture = texture;;
	setPosition(position);
	previousDirection = 0;
	direction = 0;
	frame = 0;
	currentFrame = 0;
	velocity = sf::Vector2f(0,0);
	rotation = -45.0;
	rotations = 0;
	inWater = true;
	accelerating = false;
	decelerating = false;
	sprite.setTexture(texture);
	sprite.setTextureRect(sf::IntRect(0,0,200,200));
	int spriteX, spriteY;
	spriteX = sprite.getLocalBounds().width;
	spriteY = sprite.getLocalBounds().height;
	sprite.setOrigin(spriteX/2, spriteY/2);
	sprite.setRotation(rotation);
}

sf::Vector2f Player::getPosition()
{
	return position;
}

void Player::setPosition(sf::Vector2f position)
{
	this->position = position;
	sprite.setPosition(position);
}

float Player::getSpeed()
{
	return sqrt(velocity.x*velocity.x + velocity.y*velocity.y);
}

float Player::getRotations()
{
	return rotations;
}

void Player::increaseSpeed()
{
	accelerating = true;
}

void Player::decreaseSpeed()
{
	decelerating = true;
}

void Player::left(sf::Time time)
{
	rotate(-1, time);
	direction = 2;
}

void Player::right(sf::Time time)
{
	rotate(1, time);
	direction = 1;
}

void Player::forward()
{
	direction = 0;
}

float normalizeAngle(float num)
{
	while(num < 0)
	{
		num += 360;
	}
	while(num >= 360)
	{
		num -= 360;
	}
	return num;
}

void Player::rotate(int direction, sf::Time time)
{
	float water_rotation = 0.0001;
	float air_rotation = 0.000225;

	if(isUnderwater())
	{
		rotation += ((float)direction * water_rotation * (float)time.asMicroseconds());
	}
	else
	{
		rotation += ((float)direction * air_rotation * (float)time.asMicroseconds());
		rotations += ((float)direction * air_rotation * (float)time.asMicroseconds());
	}
	sprite.setRotation(rotation);
	if(isUnderwater())
	{
		if(isInControl())
		{
			float totalSpeed = getSpeed();
			velocity.x = cos(rotation*M_PI/180)*totalSpeed;
			velocity.y = sin(rotation*M_PI/180)*totalSpeed;
		}
	}
}

bool Player::isInControl()
{
	if(abs(normalizeAngle(movementDirection()) - normalizeAngle(rotation)) < 40.0
		|| abs(normalizeAngle(movementDirection()) - normalizeAngle(rotation)) > 320.0)
	{
		return true;
	}
	return false;
}

bool Player::isUnderwater()
{
	return position.y >= 0;
}

float Player::movementDirection()
{
	float movementDirection = (180/M_PI)*atan(velocity.y/velocity.x);
	if(velocity.x < 0 && velocity.y > 0) movementDirection -= 180;
	if(velocity.x < 0 && velocity.y < 0) movementDirection -= 180;
	if(velocity.x > 0 && velocity.y < 0) movementDirection += 360;
	return movementDirection;
}

void Player::move(sf::Vector2f movement)
{
	position += movement;
	sprite.setPosition(position);
}

namespace patch
{
    template < typename T > std::string to_string( const T& n )
    {
        std::ostringstream stm ;
        stm << n ;
        return stm.str() ;
    }
}

void Player::tick(sf::Time timeSinceLastTick, std::vector<std::pair<std::string, sf::Time>>& messages, std::vector<std::pair<std::string, sf::Time>>& messages2, int& score)
{
	if(isUnderwater())
	{
		if(inWater == false) //just landed
		{
			int totalScore = 1;
			std::string trickMessage;
			if(isInControl())
			{
				trickMessage += "Perfect ";
				totalScore = 3;
			}
			else
			{
				trickMessage += "Poor ";
			}
			if(rotations >= 780)
			{
				trickMessage += "double frontflip!";
				totalScore *= 400;
			}
			else if(rotations >= 400)
			{
				trickMessage += "frontflip!";
				totalScore *= 100;
			}
			else if(rotations <= -780)
			{
				trickMessage += "triple backflip!";
				totalScore *= 500;
			}
			else if(rotations <= -400)
			{
				trickMessage += "double backflip!";
				totalScore *= 250;
			}
			else if(rotations <= -60)
			{
				trickMessage += "backflip!";
				totalScore *= 150;
			}
			else
			{
				trickMessage += "landing!";
				totalScore *= 50;
			}
			score += totalScore;
			messages2.push_back(std::make_pair(patch::to_string(totalScore) + " points!", sf::Time()));
			messages.push_back(std::make_pair(trickMessage, sf::Time()));
		}
		inWater = true;
		rotations = 0;
	}
	else
	{
		inWater = false;
	}
	if(isUnderwater())
	{
		float acceleration = 0.00000000025;
		float deceleration = 0.00000000005;
		if(!isInControl() && getSpeed() > 0.00068)
		{
			accelerating = false;
			decelerating = true;
			deceleration = 0.0002;
		}
		float totalSpeed = getSpeed();
		if(accelerating)
		{
			frame+=(float)timeSinceLastTick.asMicroseconds();
			if(totalSpeed <= 0.0017)
			{
				velocity.x += cos(rotation*M_PI/180)*acceleration*(float)timeSinceLastTick.asMicroseconds();
				velocity.y += sin(rotation*M_PI/180)*acceleration*(float)timeSinceLastTick.asMicroseconds();
			}
		}
		if(decelerating)
		{
			velocity.x /= 1+0.000001*(float)timeSinceLastTick.asMicroseconds();
			velocity.y /= 1+0.000001*(float)timeSinceLastTick.asMicroseconds();
		}
	}
	else
	{
		float gravity = 0.00000000085;
		velocity.y += gravity*(float)timeSinceLastTick.asMicroseconds();
	}
	move(velocity * (float)timeSinceLastTick.asMicroseconds());
	accelerating = false;
	decelerating = false;

	int animationFrame = 4;
	if(isUnderwater())
	{
		//float currentFrameRate = (350.0-100*getSpeed())*8.0*(float)timeSinceLastTick.asMicroseconds();
		float currentFrameRate = (1000000 - 800000*(getSpeed()/0.0017));
		int frames = (int)(frame / currentFrameRate);
		currentFrame = (currentFrame + frames) % 4;
		frame -= frames*currentFrameRate;
		animationFrame = currentFrame;
	}
	std::vector<int> frameList = {0,1,0,2,3};
	sprite.setTextureRect(sf::IntRect(200*frameList.at(animationFrame), 200*direction, 200, 200));
	float colorMultiplier = 1;
	if(position.y < 1500 && position.y > 0)
	{
		colorMultiplier = (position.y/1500.0f);
	}
	else if(position.y <= 0)
	{
		colorMultiplier = 0;
	}
	sprite.setColor(sf::Color(255.0f*colorMultiplier,255.0f*colorMultiplier,255.0f*colorMultiplier));
	previousDirection = direction;
}

void Player::draw(sf::RenderTarget &target, sf::RenderStates states)const
{
	target.draw(sprite, states);
}
